unit FConverter;


interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls,
  novaOptions, novapiLib90_TLB,
  novaMessages;

type
  TForm1 = class(TForm)
    cbProfiles: TComboBox;
    Label1: TLabel;
    gbOptions: TGroupBox;
    cbCompress: TCheckBox;
    cbEmbed: TCheckBox;
    cbDetect: TCheckBox;
    gbDocInfo: TGroupBox;
    lblTitle: TLabel;
    lblSubject: TLabel;
    efTitle: TEdit;
    efSubject: TEdit;
    gbPDFFile: TGroupBox;
    Label4: TLabel;
    efFileName: TEdit;
    gbFileToConvert: TGroupBox;
    efFileToConvert: TEdit;
    btnBrowse: TButton;
    OpenDialog: TOpenDialog;
    btnStartPrint: TButton;
    btnCancel: TButton;
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure btnStartPrintClick(Sender: TObject);
    procedure cbProfilesSelect(Sender: TObject);
    procedure btnBrowseClick(Sender: TObject);
    procedure btnCancelClick(Sender: TObject);
  private
    { Private declarations }
    m_novaOptions : INovaPdfOptions90;
    m_bPrintJobPending : BOOLEAN;
    m_strSmallSizeProfileID : WideString;
    m_strFullOptProfileID : WideString;

    procedure CreateProfiles;
    procedure LoadProfiles;
    procedure UpdateDialogFromProfile(p_strProfileId : WideString);
    procedure UpdateProfileFromDialog(p_strProfileId : WideString);
    procedure MyExceptionHandler(Sender : TObject; E : Exception);
  public
    { Public declarations }
    procedure WndProc(var Message: TMessage); override;
  end;

TProfileObj = class(TObject)
  ProfileId: String;
end;

var
  Form1: TForm1;

implementation


uses
  ActiveX,
  novaErrors,
  ShellAPI,
  ComObj;


var
  hr : HRESULT;


const
  PRINTER_NAME       = 'novaPDF SDK 9';
  SMALL_SIZE_PROFILE = 'Small Size Profile';
  FULL_OPT_PROFILE   = 'Full Options Profile';
  PROFILE_IS_PUBLIC  = 0;


{$R *.dfm}

procedure TForm1.FormCreate(Sender: TObject);
begin
  // register event messages
  WM_NOVAPDF2_FILESAVED  := RegisterWindowMessage(MSG_NOVAPDF2_FILESAVED);
  WM_NOVAPDF2_PRINTERROR := RegisterWindowMessage(MSG_NOVAPDF2_PRINTERROR);

  // do not show error message for exceptions from COM
  Application.OnException := MyExceptionHandler;

  m_bPrintJobPending := FALSE;

  // initialize COM libraries
  hr := ActiveX.CoInitialize(nil);
  if FAILED(hr) then begin
    MessageDlg('Failed to initialize COM' +#13+SysErrorMessage(hr) +#13+
               SysErrorMessage(GetLastError()), mtWarning, [mbOK], 0);
  end;

  //create an instance of INovaPdfOptions
  m_novaOptions := nil;
  hr := ActiveX.CoCreateInstance(CLASS_NovaPdfOptions90, //CLSID_CNovaPdfSource,
                                 nil,
                                 CLSCTX_INPROC_SERVER,
                                 IID_INovaPdfOptions90,
                                 m_novaOptions);
  if (FAILED(hr))	then begin
    MessageDlg('Failed to create novaPDF COM object', mtWarning, [mbOK], 0);
	exit;
  end;

  // initialize the NovaPdfOptions object to use with a printer licensed for SDK
  // if you have an application license for novaPDF SDK, 
  // pass the license key to the Initialize() function
  m_novaOptions.Initialize( PRINTER_NAME, '' );
  if (FAILED(hr))	then begin
    MessageDlg('Failed to initialize NovaPdfOptions', mtWarning, [mbOK], 0);
	exit;
  end;

  // add 2 profiles
  CreateProfiles();

  // load profiles in list
  LoadProfiles();

end;

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
var
  bClose : BOOLEAN;
begin
  bClose := FALSE;
  if (m_bPrintJobPending) then begin
    if (IDYES = MessageDlg('A print job is pending. '+
                           'Are you sure you want to close the application?',
                           mtWarning, [mbYes, mbNo], 0)) then begin
      bClose := TRUE;
    end;
  end else begin
    bClose := TRUE;
  end;
  if bClose then begin
    //free profiles objects added in the combo-box
    While (cbProfiles.Items.Count > 0) do begin
      if Assigned(cbProfiles.Items.Objects[cbProfiles.Items.Count -1]) then begin
        TProfileObj(cbProfiles.Items.Objects[cbProfiles.Items.Count -1]).Free();
      end;
      cbProfiles.Items.Delete(cbProfiles.Items.Count -1);
    end;

    //delete profiles
    m_novaOptions.DeleteProfile2( m_strSmallSizeProfileID );
    m_novaOptions.DeleteProfile2( m_strFullOptProfileID );

    // destroy m_novaOptions object
    //  - no need for this as the Delphi takes care of it automatically

    // uninitialize COM libraries
    ActiveX.CoUninitialize();

    Action := caFree;
  end else begin
    Action := caNone;
  end;
end;

procedure TForm1.CreateProfiles();
begin

    // Add a profile called "Small size"
	m_novaOptions.AddProfile2(SMALL_SIZE_PROFILE, PROFILE_IS_PUBLIC, m_strSmallSizeProfileID);
	m_novaOptions.LoadProfile2(m_strSmallSizeProfileID);

    // Set some options to this profile

    // disable the "Save PDF file as" prompt
    m_novaOptions.SetOptionLong(NOVAPDF_SAVE_PROMPT_TYPE, PROMPT_SAVE_STNANDARD);
    // set generated Pdf files destination folder to the application path
    m_novaOptions.SetOptionLong(NOVAPDF_SAVE_FOLDER_TYPE, SAVEFOLDER_CUSTOM);
    m_novaOptions.SetOptionString2(NOVAPDF_SAVE_FOLDER,
                                       ExtractFilePath(Application.ExeName));
    // set output file name
    m_novaOptions.SetOptionString2(NOVAPDF_SAVE_FILE_NAME, 'PDF Converter small size.pdf');
    // if file exists in the destination folder, append a counter to the end of the file name
    m_novaOptions.SetOptionLong(NOVAPDF_SAVE_FILEEXIST_ACTION,
                                     FILE_CONFLICT_STRATEGY_AUTONUMBER_NEW);
    // don't detect URLs
    m_novaOptions.SetOptionBool(NOVAPDF_URL_ANALIZE, 0);

    // Set image compresion method to JPEG and quality to 75, possible values are from 10 to 100
    m_novaOptions.SetOptionBool(NOVAPDF_COMPRESS_ENABLE, 1);
    m_novaOptions.SetOptionLong(NOVAPDF_COMPRESS_HIGH_COLOR_TYPE, COMPRESS_METHOD_JPEG);
    m_novaOptions.SetOptionLong(NOVAPDF_COMPRESS_HIGH_COLOR_LEVEL, 75);

    // make sure text compression is enabled, and set compression level to 9 (maximum)
    // (posible values are 1-9)
    m_novaOptions.SetOptionBool(NOVAPDF_COMPRESS_TEXT_GRAPHICS, 1);
    m_novaOptions.SetOptionLong(NOVAPDF_COMPRESS_TEXT_GRAPHICS_LEVEL, 9);

    // disable unused font embeding
    m_novaOptions.SetOptionBool(NOVAPDF_FONTS_EMBED_ALL_USED_FONTS, 0);

    //save the profile
    m_novaOptions.SaveProfile();

    // Add the profile "Full options", and edit its options
    m_novaOptions.AddProfile2(FULL_OPT_PROFILE, PROFILE_IS_PUBLIC, m_strFullOptProfileID);
	m_novaOptions.LoadProfile2(m_strFullOptProfileID);

    // disable the "Save PDF file as" prompt
    m_novaOptions.SetOptionLong(NOVAPDF_SAVE_PROMPT_TYPE, PROMPT_SAVE_STNANDARD);
    // set generated Pdf files destination folder to the application path
    m_novaOptions.SetOptionLong(NOVAPDF_SAVE_FOLDER_TYPE, SAVEFOLDER_CUSTOM);
    m_novaOptions.SetOptionString2(NOVAPDF_SAVE_FOLDER,
                                       ExtractFilePath(Application.ExeName));
    // set output file name "[N] full", the [N] macro means document name
    m_novaOptions.SetOptionString2(NOVAPDF_SAVE_FILE_NAME, '[N] full');
     // if file exists in the destination folder, append a counter to the end of the file name
    m_novaOptions.SetOptionLong(NOVAPDF_SAVE_FILEEXIST_ACTION,
                                     FILE_CONFLICT_STRATEGY_AUTONUMBER_EXIST);
    // enable URL detection
    m_novaOptions.SetOptionBool(NOVAPDF_URL_ANALIZE, 1);

    // set image JPEG quality to 100 (maximum)
    m_novaOptions.SetOptionBool(NOVAPDF_COMPRESS_HIGH_COLOR, 1);
    m_novaOptions.SetOptionLong(NOVAPDF_COMPRESS_HIGH_COLOR_TYPE, COMPRESS_METHOD_JPEG);
    m_novaOptions.SetOptionLong(NOVAPDF_COMPRESS_HIGH_COLOR_LEVEL, 100);

    // set text compression to 6 (on the "zip" scale from 0 to 9)
    m_novaOptions.SetOptionBool(NOVAPDF_COMPRESS_TEXT_GRAPHICS, 1);
    m_novaOptions.SetOptionLong(NOVAPDF_COMPRESS_TEXT_GRAPHICS_LEVEL, 6);
    // enable font files embedding
    m_novaOptions.SetOptionBool(NOVAPDF_FONTS_EMBED_ALL_USED_FONTS, 1);

    //save the profile
    m_novaOptions.SaveProfile();

end;

procedure TForm1.LoadProfiles();
var
  strProfileId       : WideString;
  strProfileName     : WideString;
  strActiveProfileId : WideString;
  Obj                : TProfileObj;
  i                  : Integer;
begin
  // Fill profiles combo box with profile names
  m_novaOptions.GetFirstProfile2(strProfileId);
  while Length(strProfileId) > 0 do begin
	m_novaOptions.LoadProfile2(strProfileId);
	m_novaOptions.GetOptionString2(NOVAPDF_PROFILE_NAME, strProfileName);
    // add profile name and id to combo-box
    Obj := TProfileObj.Create();
    Obj.ProfileId := strProfileId;
    cbProfiles.AddItem(strProfileName, Obj);
    try
      // get next profile if it exists; exception raised when no more profiles
      m_novaOptions.GetNextProfile2(strProfileId);
    except
      on E: EOLEException do
      begin      //no more profiles
        break;
      end;
    end;
  end;
  // select the active profile in the profiles combo box
  try
    m_novaOptions.GetActiveProfile2(strActiveProfileId);
  except
  end;
  for i := 0 to cbProfiles.Items.Count - 1 do begin
    Obj := TProfileObj(cbProfiles.Items.Objects[i]);
    if Obj.ProfileId = strActiveProfileId then begin
      cbProfiles.ItemIndex := i;
      break;
    end;
  end;
  //if active profile not found, select first profile
  if cbProfiles.ItemIndex < 0 then begin
    cbProfiles.ItemIndex := 0;
  end;
  //ubdate controls with selected profile
  UpdateDialogFromProfile(TProfileObj(cbProfiles.Items.Objects[cbProfiles.ItemIndex]).ProfileId);
end;

procedure TForm1.UpdateDialogFromProfile(p_strProfileId : WideString);
var
  nVal      : LongInt;
  strOption : WideString;
begin
  // update dialog data with the data from the profile
  nVal := 0;

  m_novaOptions.LoadProfile2(p_strProfileId);

  m_novaOptions.GetOptionBool( NOVAPDF_COMPRESS_TEXT_GRAPHICS, nVal);
  cbCompress.Checked := Boolean(nVal);

  m_novaOptions.GetOptionBool(NOVAPDF_URL_ANALIZE, nVal);
  cbDetect.Checked := Boolean(nVal);

  m_novaOptions.GetOptionBool(NOVAPDF_FONTS_EMBED_ALL_USED_FONTS, nVal);
  cbEmbed.Checked := Boolean(nVal);

  strOption := '';
  m_novaOptions.GetOptionString2(NOVAPDF_SAVE_FILE_NAME, strOption);
  efFileName.Text := strOption;

  m_novaOptions.GetOptionString2(NOVAPDF_DOCINFO_SUBJECT, strOption);
  efSubject.Text := strOption;

  m_novaOptions.GetOptionString2(NOVAPDF_DOCINFO_TITLE, strOption);
  efTitle.Text := strOption;

end;

procedure TForm1.UpdateProfileFromDialog(p_strProfileId : WideString);
begin
	
  m_novaOptions.LoadProfile2(p_strProfileId);

  m_novaOptions.SetOptionBool(NOVAPDF_COMPRESS_TEXT_GRAPHICS, LongInt(cbCompress.checked));
  m_novaOptions.SetOptionBool(NOVAPDF_URL_ANALIZE, LongInt(cbDetect.checked));
  m_novaOptions.SetOptionBool(NOVAPDF_FONTS_EMBED_ALL_USED_FONTS, LongInt(cbEmbed.Checked));
  m_novaOptions.SetOptionString2(NOVAPDF_SAVE_FILE_NAME, efFileName.text);
  m_novaOptions.SetOptionString2(NOVAPDF_DOCINFO_SUBJECT, efSubject.Text);
  m_novaOptions.SetOptionString2(NOVAPDF_DOCINFO_TITLE, efTitle.text);

  m_novaOptions.SaveProfile();
end;

procedure TForm1.btnStartPrintClick(Sender: TObject);
var
  hExec : HINST;
  strProfileId : WideString;
begin
  if (m_bPrintJobPending) then begin
    MessageDlg('A print job is pending', mtWarning, [mbOK], 0);
    Exit;
  end;

  //get selected profile Id
  strProfileId := TProfileObj(cbProfiles.Items.Objects[cbProfiles.ItemIndex]).ProfileId;

  // save settings from dialog to profile
  UpdateProfileFromDialog(strProfileId);

  // check to see if there is a file to convert
  if NOT FileExists(efFileToConvert.Text) then begin
    // no file to convert
    MessageDlg('File to convert not found !', mtWarning, [mbOK], 0);
    exit;
  end;

  // set the active profile to be used for printing
  m_novaOptions.SetActiveProfile2(strProfileId);

  // register our window to receive messages from the printer
  m_novaOptions.RegisterEventWindow(self.Handle);

  // set novaPDF as default printer, so it will be used by ShellExecute
  m_novaOptions.SetDefaultPrinter();

  // license the file to be converted with ShellExecute
  m_novaOptions.LicenseShellExecuteFile(efFileToConvert.Text);

  // uncomment this code if you have a 32 bit app installed on a 64 bit OS in windows for every print job // 
  //m_novaOptions.LicenseApplication('SPLWOW64.EXE');
  
  // print the document
  m_bPrintJobPending := TRUE;

  hExec := ShellAPI.ShellExecute(self.handle, 'print', PChar(efFileToConvert.Text),
                                 PChar(''), PChar(''), SW_HIDE);

  if (hExec <= 32) then begin // failed to execute program
    m_bPrintJobPending := FALSE;
    m_novaOptions.UnRegisterEventWindow();
    m_novaOptions.RestoreDefaultPrinter();
  end;

end;

procedure TForm1.btnCancelClick(Sender: TObject);
begin
  Close();
end;

procedure TForm1.cbProfilesSelect(Sender: TObject);
begin
  UpdateDialogFromProfile(TProfileObj(cbProfiles.Items.Objects[cbProfiles.ItemIndex]).ProfileId);
end;

procedure TForm1.btnBrowseClick(Sender: TObject);
begin
  if OpenDialog.Execute() then begin
    efFileToConvert.Text := OpenDialog.FileName;
  end;
end;

procedure TForm1.WndProc(var Message: TMessage);
begin
  if Message.Msg = WM_NOVAPDF2_FILESAVED then begin

    // restore original default printer
    m_novaOptions.UnRegisterEventWindow();
    m_novaOptions.RestoreDefaultPrinter();
    m_bPrintJobPending := FALSE;

  end else if Message.Msg = WM_NOVAPDF2_PRINTERROR then begin

    case (Message.WParam) of
      ERROR_MSG_TEMP_FILE : begin
        MessageDlg('Error saving temporary file on printer server', mtWarning, [mbOK], 0);
      end;
      ERROR_MSG_LIC_INFO : begin
        MessageDlg('Error reading license information', mtWarning, [mbOK], 0);
      end;
      ERROR_MSG_SAVE_PDF : begin
        MessageDlg('Error saving PDF file', mtWarning, [mbOK], 0);
      end;
      ERROR_MSG_JOB_CANCELED : begin
        MessageDlg('Print job was canceled', mtWarning, [mbOK], 0);
      end;
    end;
    // restore original default printer
    m_novaOptions.UnRegisterEventWindow();
    m_novaOptions.RestoreDefaultPrinter();
    m_bPrintJobPending := FALSE;

  end else begin

    inherited WndProc(Message);

  end;
end;

procedure TForm1.MyExceptionHandler(Sender : TObject; E : Exception );
begin
if E is EOLEException  then
  //do not show message from COM;
  //there are raised exceptions for all return values different then S_OK
else
  Application.ShowException( E );
end;

end.
